
#package Commands;
# We can't explicitly say it's the package Commands because
# we want to keep the main namespace. :/

=pod 

=head1 Ymusk Commands

These commands can be either be called from the input
input window, or used in a configuration file.

=begin hacking

This is were all \commands are defined. If you want to add
your own \command, create your function and add an entry to
the %::commands hash. For example:

	$::commands{ stuff } = \&Commands::stuff;

	sub stuff{ ymusk_message( "Command 'stuff' invoked );

The subroutine receives whatever follow the command in a 
single string. 

	sub stuff
	{
		my $arg = shift;
		ymusk_message( "command stuff caleld with arguments '$arg'" );
	}

=end hacking

=head2 Commands

=over 4

=cut

############################################################

=item B</eval> I<expression>

	/eval 1 + 2/3   # a kick'n'dirty calculator

Evaluate the Perl expression and print its result in the 
output window (the output is not sent to the mush). 
The expression is evaluated in the script's namespace, 
which is a nifty hacking backdoor.

=cut

$commands{eval} = sub{ 
	if( my $o = eval shift )
    { 
		ymusk_message( $o ) 
	}
   ymusk_message( $@ ) if $@;
};

$commands{ quit } = sub{ exit };

############################################################

=item B</help> [ I<command> ]

	/help
	/help help
	/help eval

Without argument, lists all commands in the output window.
If I<command> is given, gives the documentation of this particular
command.

=cut

$commands{ help } = \&Commands::help;

sub Commands::help
{
	if( $_[0] )
	{
		unless( $commands{ $_[0] } )
		{
			ymusk_message( "command $_[0] not recognized" );
			return;
		}

		open MYSELF, __FILE__;
		my $ok;
		while(<MYSELF>)
		{
			if( m#^=item B</$_[0]># .. ( $_[1] || /^=cut/ ) )
			{
				$ok = 1;
				s/^=\w+//;
				s/[B]<(.*?)>/$1/g;
				s/[I]<(.*?)>/<$1>/g;
				s/\t/    /g;
				ymusk_message( $_ );
			}
		}
		close MYSELF;
		ymusk_message( "nothing found for command /$_[0]" ) unless $ok;
	}
	else
	{
		ymusk_message( join ' ', sort( keys %commands ) );	
	}

	return 1;
}				  

############################################################

=item B</usage> I<command>

	/usage help
	/usage margin

Gives argument list of I</command>. For more details,
use B</help>.

=cut

$commands{ usage } = sub{ Commands::help( @_, 1 ); };

############################################################

$commands{ hook } = \&Commands::hook;
sub Commands::hook
{
	$_[0] =~ /(\S.*?)\s*=\s*(.*)/; 
	Commands::help( 'hook', 1 ) and return unless $1 and $2;
	$hooks{ eval $1 } = eval "sub{ $2 ; return () }";
}

############################################################

=item B</bind> <I<key>> = I<command>
 
	/bind <F1> = &type( "@mail" )

Bind <I<key>> to I<command> in the input window. 
See Perl/tk documentation for details on key binding
  
Two functions of ymusk are really, really useful for I<command>:
I<&type> and I<&send>. The difference is that I<&type>
is preprocessed by ymusk (so recognize /commands and the like)
while I<&send> send the string directly to the mush server. Unless
you want to do some black magic, use I<&type>. 
   
=cut

$commands{ bind } = \&Commands::bind;

sub Commands::bind
{
	$_[0] =~ /(<.*?>)\s*=\s*(.*)/; 
	ymusk_message( "Bad bind command: $_[0]" ) and return unless $1 and $2;
	$input->bind( $1, eval "sub{ $2 }" );
}

############################################################

=item B</do> I<file>

	/do myscript.txt

Executes I<file> as if entered by the 
user in the input window, line by line.

=cut 

$commands{ do } = \&Commands::do;

sub Commands::do 
{
	open FILE, shift or ymusk_message( $! ) and return;
	foreach( <FILE> )
	{
		$input->insert( 'end', $_ );
		parse_input();
	}
	ymusk_message( "File parsed" );

	close FILE;
}

############################################################


=item B</connect> I<host> I<port> [ I<character> I<password> ]

Connect to I<host> at given I<port>, optionaly logging in 
as I<character> using I<password>.

=cut

$commands{ connect } = \&Commands::connect;

sub Commands::connect
{
   my @params = split ' ', shift;
   
   unless( ( @params == 2 ) or ( @params == 4 ) )
   {
     ymusk_message( '/connect host port [ character password ]' );
	 return;
   }

   my ( $host, $port, $password );
   ( $host, $port, $character, $password ) = @params;

	# close previous connection (if any)
   $telnet->close();
   $telnet->host( $host );
   $telnet->port( $port );

   $telnet->errmode( \&ymusk_message ); # just for the open()
   $telnet->errmsg();
   $telnet->open();
   $telnet->errmode( 'return' );

	return if $telnet->errmsg();

   if( $character ){ &send( "connect $character $password\n" ); }
   
   start_log( $character || $host );

   # change the main window title
   $mw->configure( -title => "Ymusk - " . ( $character || $host ) );

   $time_online = time;
}

############################################################

=item B</disconnect>

	/disconnect

Disconnect from the current host.

=cut

$commands{ disconnect } = \&Commands::disconnect;

sub Commands::disconnect
{ 
	$telnet->close(); 
	ymusk_message( "connection with " . $telnet->host() ." closed" );
	$mw->configure( -title => 'Ymusk' );
	$time_online = 0;
	$character = '';
}	

############################################################

=item B</margin> I<offset>

	/margin 20		# set margin to 20 pixels
	/margin +10 	# increase margin of 10 pixels

Set the left margin of wrapped lines. 
I<Offset> is in pixel. It can be an absolute value or can
be relative to the current offset. Also, see 
<Ctl-minus> and <Ctl-equal>.

=cut

$commands{ margin } = \&Commands::margin;

sub Commands::margin
{
	my $size = shift;
	return unless $size =~ /[+-]?\d+/;

	foreach my $output( $output, values %output )
	{
		if( $size =~ /[+-]/ )
		{
			$size += $output->tagCget( 'margin' => '-lmargin2' );
			$size = 0 if $size < 0
		}

		$output->tagConfigure( 'margin', -lmargin2 => $size );
		$output->tagAdd( 'margin', '1.0', 'end' );
	}
}

############################################################

=item B</paragraph> I<offset>

	/paragraph 10 
	/paragraph +5

Set the spacing between paragraphs. It can be an absolute value or
be relative to the current spacing. Also, see 
<Ctl-underscore> and <Ctl-plus>.

=cut

$commands{ paragraph } = \&Commands::paragraph;
sub Commands::paragraph
{
    my $size = shift;
    return unless $size =~ /[+-]?\d+/;

    if( $size =~ /[+-]/ )
    {
        $size += $output->cget( '-spacing3' );
        $size = 0 if $size < 0
    }

	foreach my $output( $output, values %output )
	{
		$output->configure( -spacing3 => $size );
	}
}

############################################################

=item B</tag> I<tag> = ( I<-option> => I<-value> )

Redefine the text tag I<tag>. To know the tags already recognized by ymusk,
you can do 

	/eval join " ", keys %tags

Of course, new tags can be added to fit any particular need. if you want
to such a thing, do peek at B<Hooks.pm>. Which make me think I 
should begin to write a Hacking guide. Hmmm....

=cut

$commands{ tag } = \&Commands::tag;
sub Commands::tag
{
	$_[0] =~ /(\S.*?)\s*=\s*(.*\S)/; 
	ymusk_message( "Bad tag command: $_[0]" ) and return unless $1 and $2;
	$output->tagConfigure( $1, eval( $2 ) );
	
}

############################################################

=item B</log> [ B<off> | I<logfile> ]

Log mush output to I<logfile>. 
If I<logfile> doesn't exist, it is created. 
If it does exist, the log is appended
to the file. If the parameter B<off>
is given, ymusk stops to log. If no parameter is passed,
the current log file is returned. 

=cut

$commands{ log } = \&Commands::log;

sub Commands::log
{
	my @params = split ' ', shift;
	
	if( @params == 0 )
	{
		if( $logfile )
		{
			ymusk_message( "logging in $logfile" );
		}
		else
		{
			ymusk_message( "log not enabled" )
		}
	}
	elsif( @params > 1 )
	{
		ymusk_message( "/log [ off|<filename>]\n" )	
	}
	elsif( $params[0] =~ /^off$/i )
	{
		close LOG;
		$logfile = ''; 
		ymusk_message("log file closed" );
		$log_status = '';
	}
	else
	{
		$logfile = $params[0];
		if( open LOG, ">>$logfile" )
		{
			ymusk_message( "logging at $logfile" );
			$log_status = 'L';
		}
		else
		{
			ymusk_messsage( "error in opening log file ($!)" );
		}
	}
}

############################################################

$commands{ 'eliza' } = \&Commands::eliza;
use vars qw/ $eliza /;

=item /eliza [ I<on> ]

	/eliza on     # enable Eliza
	/eliza        # disable Eliza

Create an Eliza psychoanalyst that will answer IC
dialogue in your place. Eliza is smart enough to 
to reply to herself or to reply to pages and <Channel> 
stuff. Require the module Chatbot::Eliza.

=cut
	
sub Commands::eliza
{
	if( shift )
	{
		eval { require Chatbot::Eliza };
		if( $@ )
		{
			ymusk_message( "require module Chatbot::Eliza" );
			return;
		}
		$eliza = new Chatbot::Eliza( "Eliza" );
		$eliza->memory_on( 25 );

		$hooks{ qr/^[^<].*".*"/ } = \&Hooks::eliza;
		ymusk_message( 'Eliza is enabled' );
	}
	else
	{
		delete $hooks{ qr/^[^<].*".*"/ };
		ymusk_message( 'Eliza is disabled' );
	}
}

sub Hooks::eliza
{
	my $line = shift;	
	return if $line =~ /^($character|You)/ or $line =~ /page|from afar|long distance/;

	$line = join ' ', $line=~/"(.*?)"/g;
	ymusk_message( $line . "\n" );
	&send( '"' . $eliza->transform( $line ). "\n" );

	return ();
}

############################################################

=item B</pickupline>

	/pickupline

Generate a random pickup line for your character. Those 
pickup lines can be obscene, raunchy, offensive and/or
nonsensical. Furthermore, just to make things more
dangerous, the pickup line will be emitted directly
to the mush, so you will discover it at the same time
as everybody else. In other words, if your character
get lynched because of this command, don't put the blame on 
us. Require the Easy::Bone module.

=cut

$commands{ pickupline } = \&Commands::pickupline;
sub Commands::pickupline
{
	eval { require Bone::Easy };
	if( $@ )
	{
		ymusk_message( "require module Bone::Easy" );
		return;
	}
	&send( '"'. Bone::Easy::pickup() . "\n" );	
}

=item B</tz> I<@timezones>

	/tz EST
	/tz szasha anastasia cyrano

Return the current time in another timezone. The argument
can be a timezone or a label registred with /tz_add.

=cut

$commands{ tz } = \&Commands::tz;

sub Commands::tz
{
	eval {require Time::Timezone};

	ymusk_message( "module Time::Timezone required for command /tz" )
		and return if $@;

	foreach( split ' ', shift )
	{
		ymusk_message( $_ . " => " 
						  . gmtime( time + tz_offset( $tz{$_} || $_  ) ) );
	}
}

=item B<tz_add> I<$label> = I<$timezone>

	/tz_add  szasha = EST

Add a timezone label. See /tz

=cut

$commands{ tz_add } = \&Commands::tz_add;
sub Commands::tz_add
{
	if( shift =~ /(\w+)\s*=\s*(\w+)/ )
	{
		$tz{$1} = $2;
	}
	else
	{
		ymusk_message( "Error" );
	}
}

=item B<tz_list>

	/tz_list

List all timezone labels in memory.

=cut

$commands{ tz_list } = \&Commands::tz_list;

sub Commands::tz_list
{
	ymusk_message( join " ", map "$_ ($tz{$_})", keys %tz );	
}

############################################################
=item B</date>

	/date create repeat mm hh dd MM yyyy <Text>
	/date erase <date id.>

	ie:
	/date create 15d 10 4 12 12 2000 Pay day!
		(creates one alarm at 4:10 am on the 12 of December 2000
		that will be repeated every 15 at the same hour days till then)
	/date create 1M 10 4 12 12 2000 Has past exactly one month!
	/date create 1m 10 4 12 12 2000 Has past exactly one minute!
	 
Creates/deletes/list dates.
With no parameter lists all the dates.
Dates can be anything you want toremember at a given time. When 
it is the time you specified or it has passed you will be shown all the
dates that has past (useful if the date comes when you are not
running ymusk) or are comming next. An id number will be added
auto to each date for easy locate/delete the entry. All the dates will
be saved at dates.ymusk file into ymusk dir.
REMEMBER TO TYPE A VALID DATE ymusk won't say anything
if you type an alarm for the 31 of February! ;) (although ymusk will show
your alarm on the 1 of March... how good is it! ;)

=cut

$commands{ 'date' } = \&Commands::date;
sub Commands::date
{
	my @params = split ' ', shift;

	#list of dates
	if ( @params == 0 )
	{

		# No file? Ah, ok...
		return unless -e "$ENV{HOME}/ymusk/dates.ymusk";

		open DATES, "$ENV{HOME}/ymusk/dates.ymusk" or 
			die "Can't open file $ENV{HOME}/ymusk/dates.ymusk: $!\n";
		
		$id="0";
		ymusk_message("-- DATES: --");
		ymusk_message("id repeat mm hh dd MM yyyy <text>");
		ymusk_message("---------------------------------");
		while (<DATES>)
		{
			#format is: id repeat mm hh dd MM yyyy <text>
			ymusk_message("$id $_");
			$id++;
		}
		close DATES;
		return;
	}
	#delete one entry
	elsif (@params == 2 && $params[0] =~ /^erase$/ && $params[1] =~ /\d*/)
	{
		open DATES, "$ENV{HOME}/ymusk/dates.ymusk" or 
			die "Can't open file $ENV{HOME}/ymusk/dates.ymusk: $!\n";
		
		@TEMP = <DATES>;
		close DATES;
			# wrong id number?
		if ($params[1] > @TEMP)
		{
			ymusk_message("Wrong id number...");
			return;
		}
			# deletes the specified line
		splice (@TEMP,$params[1],1) or	
			die "Can't find that date!, take a look again at the list";
		
			# let's write the changes deleting the old file
		open DATES, ">$ENV{HOME}/ymusk/dates.ymusk" or 
			die "Can't open file $ENV{HOME}/ymusk/dates.ymusk: $!\n";
		print DATES @TEMP;
		close DATES;
		
		ymusk_message("$params[1] date removed...");
		
		return;
	}
	# create an entry; format is: create repeat mm hh dd MM yyyy <text>
	elsif (@params > 7 && 
		$params[0] =~ /^create$/ &&
		$params[1] =~ /^\d*[mhdMy]$/ || $params[1] eq "na" &&
		$params[2] =~ /^\d*$/ && 0 <= $params[2] && $params[2] <= 59 &&
		$params[3] =~ /^\d*$/ && 0 <= $params[3] && $params[3] <= 24 &&
		$params[4] =~ /^\d*$/ && 0 <= $params[4] && $params[4] <= 31 &&
		$params[5] =~ /^\d*$/ && 1 <= $params[5] && $params[5] <= 12 &&
		$params[6] =~ /^\d*$/
		)
	{
		open DATES, ">>$ENV{HOME}/ymusk/dates.ymusk" or 
			die "Can't open file $ENV{HOME}/ymusk/dates.ymusk: $!\n";
		#elimination of "create" word
		shift @params;
		#creation of a string separated by spaces
		print DATES join " ", @params, "\n";
		close DATES;
		return;
	}
	
	# bad syntax
	ymusk_message("BAD SYNTAX... try /help date");
}

############################################################
=item  B</ycq>

	/ycq

Pops open the ycq widget (a cute icq-lookalike for pages).

=cut

$commands{ ycq } = sub { require Ycq; Ycq::start() };


#########################################################
=item B</world> <world> = host port [ character  password ] 

	/world fading = saturn.planetmud.com 4996
	/world yenz =  saturn.planetmud.com 4996 yenz hush

Creates a command I</world> that connect ymusk to the host is 
created, unless the command I</world> already exist. In other words, 
don't name any of your worlds 'margin', 'eval' or 'help'. Useful
for the configuration file.

=cut 

$commands{world} = \&Commands::world;
sub Commands::world
{
	unless( $_[0] =~ /(\w+) \s* = \s* (.*) $/x )
	{
		ymusk_message( '/word: bad syntax' );
		return;
	}

	my @params = split ' ', $2;
	ymusk_message( '/word: bad syntax' ) and return 
		unless @params == 2 or @params == 4;

	ymusk_message( 'keyword $1 is already taken' )
		and return if $commands{ $1 };


	$commands{ $1 } = eval "sub{ Commands::connect( '$2' );}";

}


#################################################################


=item B</webster> [ I<prefix> ] I<word>

	/webster +OOC defenestration
	/webster musk

Fetch the definition of I<word> at the
Myriam-Webster dictionary (www.m-w.com). 
If I<prefix> is not given, B<think> is
used.

=cut

$commands{ webster } = \&Commands::webster;
sub Commands::webster
{
	return unless length $_[0];

	$_[0] =~ /^(.*)(\b\w+)/;
	my( $emit, $word ) = ( $1, $2 );
	$emit ||= 'think';
	$emit =~ s/\s+$//;

	eval{ require LWP::Simple; };
	if( $@ )
	{
		ymusk_message( "/webster requires module LWP::Simple" );
		return;
	}

	my $def = LWP::Simple::get( "http://www.m-w.com/cgi-bin/dictionary?book=Dictionary&va=$word" );

	( $def ) = $def =~ m#(Main Entry:.*?)</form>#s;
	$def ||= "$word not found at www.m-w.com";
	$def =~ s/<.*?>//g;  # clean HTML tags
	$def =~ s/&lt;/</g;
	$def =~ s/&gt;/>/g;

	$def =~ s/(.)/ ord($1)>126?'':"$1" /eg;
	


	&send( "$emit $_" ) for split /\n/, $def;
}

#################################################################

=item B</scroll> [auto|more]

	/scroll auto

Set the scrolling mode to auto (always scrolls down as new
text arrive), more (after a page of information, display
a red 'more' at the top of the window and wait for the
user to press <tab> before scrolling down) or, if
no argument is given, disable scrolling altogether.

Also accessible through the Display menu.

Note: it is not recommended to use the 'more' mode in
conjuncture with multi-output mode.

=cut

$commands{ scroll } = \&Commands::scroll;
sub Commands::scroll
{
	my $mode = shift;
	unless( grep $mode eq $_, '', 'auto', 'more' )
	{
		ymusk_message( '/scroll: argument must be auto, more or nothing' );
		return;
	}

	$::scroll_type = $mode;
	switch_scroll_type();

	$mode ||= 'disabled';


	ymusk_message( "scrolling set to $mode" );


}


#################################################################


=item B</cursor> I<color>

	/cursor blue

Set the cursor's color to I<color>.

=cut

$commands{ cursor } = \&Commands::cursor;
sub Commands::cursor
{
	my( $color, $w ) = @_;
	$w ||= $mw;

	if( $w =~ /Text/i )
	{
		#my $tmp = ($w->cget( '-cursor' ))[0];
		#my $cursor = ref($tmp)?$$tmp[0]:$tmp;
		eval{ $w->configure( -cursor => [ 'xterm', $color ] ) };
	}

	Commands::cursor( $color, $_ ) foreach $w->children;
}

$commands{ show } = \&Commands::show;
sub Commands::show
{
	my $picture = shift;

	if( $picture =~ /^http/ )
	{
		eval{ require LWP::Simple; };
		if( $@ )
		{
			ymusk_message( "/show requires module LWP::Simple to get pictures from the web" );
			return;
		}

		my $file = LWP::Simple::get $picture;
		if( $file )
		{
			open DUMP, ">./dump";
			print DUMP $file;
			close DUMP;
			$output->imageCreate( 'end', -image => $mw->Photo( -format => 'gif', -data => $file ), -align=>'center'  );
			$output->insert( 'end', "\n" );
			return;
		}
		else
		{
			ymusk_message( "unable to fetch $picture" );
			return;
		}

	}
	elsif( -e $picture )
	{
		$output->imageCreate( 'end', -image => $mw->Photo( -file => $picture ), -align=>'center'  );
		$output->insert( 'end', "\n" );
		return;
	}

	ymusk_message( "$picture not found" );
}

$commands{ version } = \&Commands::version;
sub Commands::version
{
	ymusk_message( "this is ymusk v$VERSION" );

	eval{ require LWP::Simple; };
	unless( $@ )
	{
		ymusk_message( "querying ymusk.sourceforge.org" );
		my $archive = LWP::Simple::get( "http://ymusk.sourceforge.net/archives" );

		unless( $archive )
		{
			ymusk_message( "query failed" );
			return;
		}

		my $version = 0;

		while( $archive =~ /HREF=".*(\d+\.\d+).*"/g )
		{
			$version = $1 if $1 > $version;	
		}

		if( $version <= $VERSION )
		{
			ymusk_message( "ymusk is up-to-date" );
		}
		else
		{
			ymusk_message( "newer version of ymusk at http://ymusk.sourceforge.net ".
							"(v$version)" );
		}
		
	}
}

'end of module Commands';
