use Config;
use Cwd;
use ExtUtils::Installed;

if( $^O =~ /Win/i )   # I think ExtUtils::Installed barf on Windows...
{
	print "ouch. you run Windows\n",
	      "renaming ymusk to ymusk.pl\n";
	rename 'ymusk', 'ymusk.pl';
	print "creating configuration and configuration/log directories\n";
	mkdir 'configuration', 0755;
	mkdir 'configuration/log', 0755;
	print "installation complete, ymusk is ready to rock\n";
	print "Press enter to exit\n";
	<>;
	exit;
}

my @needed_module = qw/ Net::Telnet Tk /;

print "checking required modules\n";
my @modules = ExtUtils::Installed->new()->modules();
my $ok = 1;
foreach my $m( @needed_module )
{
	print "looking for module $m... ";
	print( ($ok*=grep $m, @modules )? "found\n":"not found\n");
}
print "modules are missing, ymusk will probably not work\n" unless $ok;


sub check_module
{
	my( $module, $message, $fatal ) = @_;
	print "looking for module $module... ";
	if( grep $module, @modules )
	{
		print "found\n";
	}
	else
	{
		print "not found -- $message\n";
		die if $fatal;
	}
}

check_module 'Linga::Ispell',
             'you will not be able to use the ispell command';

check_module 'LWP::Simple',
			 'you will not be able to use the webster command';

check_module 'Tk::FontDialog',
			 'oh well, no biggie';

check_module 'Chatbot::Eliza',
             'oh well, no biggie';

check_module 'Bone::Easy',
			 "I guess it's better that way";

my $pwd = cwd;

print "we are in $pwd\n";

print "looking where perl is... ";
my $fullperl = $Config{ 'perlpath' };
print "perl is $fullperl\n";

print "updating shebang line of ymusk and friends and 755'ing them\n";
for( qw[ ymusk utils/y_log2html ylog ] )
{
	print "updating shebang line of $_... ";
	update_command_line( $_ );
	print "done\n";
	print "making $_ executable... ";
	chmod 0755, $_ or die "can't make $_ executable: $!\n";
	print "done\n";
}


print "installation complete, ymusk is ready to rock\n";

sub update_command_line
{
	my $file = shift;
	open FILE, $file or die "can't open file $file: $!\n";
	my @lines = <FILE>;
	shift @lines if $lines[0] =~ /^#!/;
	unshift @lines, "#!$fullperl -I$pwd \n";
	close FILE;
	open FILE, ">$file" or die "can't open file $file: $!\n";
	print FILE @lines;
	close FILE;
}
